# Service Registry

This repository can help you to organise your infrastructure in a service-centric way. It is a skeleton in which you can integrate your Terraform modules to deploy services and teams. Services module is aimed to deploy Terraform pipelines per service in your CICD platform, whereas Teams module supposed to create infrastructure around teams and their access to the services they own.

Author: [Timur Bublik](timur.bublik@tier.app), TIER Mobility SE. 

## How to use

1. Create a service yaml in `services/service_configs` directory, the module expects manifests with filename pattern `service_<your_service_name>.yaml`. You can use your own structure, but be consistent and make adjustments in `services/local.tf`, otherwise see example `services/service_configs/service_foo.yaml`.
2. Create a team yaml in `teams/service_configs` directory, the module expects manifests with filename pattern `team_<your_team_name>.yaml`. You can use your own structure, but be consistent and make adjustments in `teams/local.tf`, otherwise see example `teams/team_configs/team_foo.yaml`.
3. Run `terraform init` and `terraform apply` to see how Service Registry processes your yaml manifests and shows what data would be available for your infrastructure modules.

### How to proceed

Add your own infrastructure modules for teams and services and use data available to you from `locals.tf` of every module and submodule.

## Limitations

Beware that the solution is far from being perfect. Some of limitations known to us are:

1. Make sure that Teams module does not depend on existence of resources outside Service Registry. For example, if you create Kubernetes namespaces per service inside of a pipeline that you deploy with Service Registry, then you must first register a service, run your newly created service pipeline (sourced from service repository) to create namespace and only then can a team claim ownership over this service. If you will try to register a service and claim ownership at the same time it will fail, because your Kubernetes namespace won't yet be created, but your team module will already try to create RoleBindings in there or other permission related resources.
2. Make sure to use lightweight Terraform modules or decompose Service Registry into two or more different state files to make sure your solution will scale when you have hundreds of services and teams.

## Requirements

1. Terraform ~> 1.0
2. A CICD platform that has a Terraform provider to deploy new pipelines

## Providers

No providers.

## Modules

| Name | Source | Version |
|------|--------|---------|
| <a name="module_services"></a> [services](#module\_services) | ./services | n/a |
| <a name="module_teams"></a> [teams](#module\_teams) | ./teams | n/a |

## Resources

No resources.

## Inputs

No inputs.

## Outputs

| Name | Description |
|------|-------------|
| <a name="output_services"></a> [services](#output\_services) | Maps of services |
| <a name="output_services_names"></a> [services\_names](#output\_services\_names) | List of services names |
| <a name="output_teams"></a> [teams](#output\_teams) | Map of teams |