locals {
  ### TIER Services ###
  # A map of our service configurations extracted from local files
  services = {
    for service in fileset("${path.module}", "service_configs/{service_}*.yaml") :
    yamldecode(file("${path.module}/${service}"))["service_name"]
    => yamldecode(file("${path.module}/${service}"))
  }

  # The filtered_stacks list is a list of maps. Each map describes a spacelift stack, consisting of the branch, repository and terraform_version.
  # This is only an intermediary representation in order to be able to deduplicate service configurations from services in a monorepo
  filtered_stacks = distinct([
    for service in local.services :
    {
      "branch"            = service.branch
      "repository"        = trimprefix(service.repository, "https://github.com/TierMobility/")
      "terraform_version" = service.terraform_version
    }
  ])

  # A deduplicated map of stacks that we feed to the spacelift service module
  module_stacks = {
    for stack in local.filtered_stacks :
    stack.repository => stack
  }

  services_names = [
    for service in local.services :
    service["service_name"]
  ]
}
